unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls;

type
  TFormSettings = class(TPersistent)
  private
    // Form whose properties are being stored
    FForm       : TForm;
    // Flag to say whether form should be updated during an Assign
    // Set by the constructor
    FUpdateForm : Boolean;
    // Form properties to store
    FTop        : Integer;
    FLeft       : Integer;
    FHeight     : Integer;
    FWidth      : Integer;
  public
    constructor Create(AForm : TForm;UpdateForm : Boolean);

    procedure Assign(Source : TPersistent); override;
  end;


  TFormSettingsRecall = class(TRecall)
  private
    FInternalRef : TFormSettings;
  public
    constructor Create(AForm : TForm);
    destructor  Destroy; override;
  end;


  TForm1 = class(TForm)
    PaintBox1: TPaintBox;
    RestorePositionBtn: TButton;
    SavePositionBtn: TButton;
    procedure PaintBox1Paint(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure RestorePositionBtnClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SavePositionBtnClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
  private
    FFormSettingsRecall : TFormSettingsRecall;

    procedure DefaultCursorSetting;
    procedure StoredCursorSetting;
    procedure DrawOnCanvas(ACanvas : TCanvas);
    procedure DrawOnCanvasRecall(ACanvas : TCanvas);
    procedure DrawOnCanvasFontRecall(ACanvas : TCanvas);
  public
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

{ TForm1 }

procedure TForm1.DefaultCursorSetting;
begin
  Screen.Cursor := crHourGlass;

  try
    //.. Do some stuff that takes a while
  finally
    Screen.Cursor := crDefault;
  end;
end;


procedure TForm1.StoredCursorSetting;
  var
    OldCursor : TCursor;
begin
  OldCursor     := Screen.Cursor;
  Screen.Cursor := crHourGlass;

  try
    //.. Do some stuff that takes a while
  finally
    Screen.Cursor := OldCursor;
  end;
end;


procedure TForm1.DrawOnCanvas(ACanvas : TCanvas);
  var
    OldFont : TFont;
begin
  OldFont := TFont.Create;

  try
    OldFont.Assign(ACanvas.Font);

    // Draw some text using the normal font for the canvas
    ACanvas.TextOut(20,20,'Some normal text');

    ACanvas.Font.Name := 'Garamond';
    ACanvas.Font.Color := clRed;
    ACanvas.Font.Style := [fsBold,fsItalic,fsUnderline];
    ACanvas.Font.Size  := 20;
    ACanvas.TextOut(20,35,'Some red text');

    ACanvas.Font := OldFont;
    // Draw some more text using the normal font for the canvas
    ACanvas.TextOut(20,75,'Some more normal text');
  finally
    OldFont.Free;
  end;
end;


procedure TForm1.DrawOnCanvasRecall(ACanvas: TCanvas);
  var
    OldFont : TRecall;
begin
  OldFont := TRecall.Create(TFont.Create,ACanvas.Font);

  try
    // Draw some text using the normal font for the canvas
    ACanvas.TextOut(20,20,'Some normal text');

    ACanvas.Font.Name := 'Garamond';
    ACanvas.Font.Color := clRed;
    ACanvas.Font.Style := [fsBold,fsItalic,fsUnderline];
    ACanvas.Font.Size  := 20;
    ACanvas.TextOut(20,35,'Some red text');
  finally
    OldFont.Free;
  end;

  // Draw some more text using the normal font for the canvas
  ACanvas.TextOut(20,75,'Some more normal text');
end;


procedure TForm1.DrawOnCanvasFontRecall(ACanvas: TCanvas);
  var
    OldFont : TFontRecall;
begin
  OldFont := TFontRecall.Create(ACanvas.Font);

  try
    // Draw some text using the normal font for the canvas
    ACanvas.TextOut(20,20,'Some normal text');

    ACanvas.Font.Name := 'Garamond';
    ACanvas.Font.Color := clRed;
    ACanvas.Font.Style := [fsBold,fsItalic,fsUnderline];
    ACanvas.Font.Size  := 20;
    ACanvas.TextOut(20,35,'Some red text');
  finally
    OldFont.Free;
  end;

  // Draw some more text using the normal font for the canvas
  ACanvas.TextOut(20,75,'Some more normal text');
end;


procedure TForm1.FormCreate(Sender: TObject);
begin
  FFormSettingsRecall := TFormSettingsRecall.Create(Self);
end;


procedure TForm1.FormDestroy(Sender: TObject);
begin
  FreeAndNil(FFormSettingsRecall);
end;


procedure TForm1.FormActivate(Sender: TObject);
begin
  // The form gets moved after it is created so save the state then
  FFormSettingsRecall.Store;
end;


procedure TForm1.SavePositionBtnClick(Sender: TObject);
begin
  if Assigned(FFormSettingsRecall) then begin
    FFormSettingsRecall.Store;
  end else begin
    FFormSettingsRecall        := TFormSettingsRecall.Create(Self);
    RestorePositionBtn.Enabled := True;
  end;
end;


procedure TForm1.RestorePositionBtnClick(Sender: TObject);
begin
  FreeAndNil(FFormSettingsRecall);
  RestorePositionBtn.Enabled := False;
end;


procedure TForm1.PaintBox1Paint(Sender: TObject);
begin
  DrawOnCanvasFontRecall(PaintBox1.Canvas);
end;


{ TFormSettings }

constructor TFormSettings.Create(AForm : TForm;UpdateForm : Boolean);
begin
  inherited Create;
  FForm       := AForm;
  FUpdateForm := UpdateForm;

  if Assigned(FForm) then begin
    FTop    := FForm.Top;
    FLeft   := FForm.Left;
    FHeight := FForm.Height;
    FWidth  := FForm.Width;
  end;
end;


procedure TFormSettings.Assign(Source : TPersistent);
begin
  if Source is TFormSettings then begin
    if not Assigned(FForm) then begin
      Exit;
    end;

    if FUpdateForm then begin
      FForm.SetBounds(TFormSettings(Source).FLeft,
                      TFormSettings(Source).FTop,
                      TFormSettings(Source).FWidth,
                      TFormSettings(Source).FHeight);
    end else begin
      FTop    := FForm.Top;
      FLeft   := FForm.Left;
      FHeight := FForm.Height;
      FWidth  := FForm.Width;
    end;
  end;
end;


{ TFormSettingsRecall }

constructor TFormSettingsRecall.Create(AForm : TForm);
begin
  FInternalRef := TFormSettings.Create(AForm,True);
  inherited Create(TFormSettings.Create(AForm,False),FInternalRef);
end;


destructor TFormSettingsRecall.Destroy;
begin
  inherited;
  // Must do this after inherited, as FInternalRef is accessed by the
  // TRecall destructor
  FreeAndNil(FInternalRef);
end;


end.
